xk6-browser v0.2.0 is here! :tada:

This is a minor release with a few new features, several bug fixes, stability improvements, and overall code cleanup.


## New features

- Support for HTTP Basic Authentication. ([#242](https://github.com/grafana/xk6-browser/pull/242))

  The `BrowserContext`'s `httpCredentials` option previously wasn't working ([#203](https://github.com/grafana/xk6-browser/issues/203)).

  The change also safely handles servers that do not adhere to the basic auth protocol by canceling the authorization process on the first failed attempt.

  It also deprecates `BrowserContext.setHttpCredentials`. The correct way of setting the credentials now is creating a new `BrowserContext` with credentials:

  ```javascript
  // Correct
  const context = browser.newContext({
    httpCredentials: {
      username: 'user',
      password: 'pass',
    }
  });

  // The following is deprecated
  context.setHTTPCredentials({
    username: 'user',
    password: 'pass',
  });
  ```

  If you need to change or clear credentials within the same script, you should create a new `BrowserContext`.

  See the reasoning behind this deprecation [here](https://github.com/microsoft/playwright/issues/2196) and [here](https://github.com/microsoft/playwright/pull/2763).

- Viewport insets now have the correct size on every OS. ([#185](https://github.com/grafana/xk6-browser/pull/185))

  Browser insets change depending on the operating system, and this feature adds support for adjusting insets of the viewport for Windows, Linux, and macOS.

  Note that it won't update the viewport if the browser runs in headless mode.

  See the related issues for more details: [#182](https://github.com/grafana/xk6-browser/issues/182), [#183](https://github.com/grafana/xk6-browser/issues/183).

- Blocking requests based on the remote host's name or IP via k6's [`blacklistIPs`](https://k6.io/docs/using-k6/options/#blacklist-ips) and [`blockHostnames`](https://k6.io/docs/using-k6/options/#block-hostnames) options. ([#204](https://github.com/grafana/xk6-browser/pull/204), [#205](https://github.com/grafana/xk6-browser/pull/205))

  The browser will block matching requests with an `ERR_BLOCKED_BY_CLIENT` error if the script specifies these options.

- Support for k6's [`hosts` option](https://k6.io/docs/using-k6/options/#hosts) to override DNS resolution with a custom mapping. ([#207](https://github.com/grafana/xk6-browser/pull/207)).

  This has the same effect as passing the [`host-resolver-rules` argument](https://datacadamia.com/web/browser/chrome#dns_resolver) to Chrome via [browser args](https://github.com/grafana/xk6-browser/blob/ae264530b6fbe0285ac9df9b0f4a907b460bb805/examples/browser_args.js).


## Bugs fixed

- `Page.queryAll()` and `Element.queryAll()` not finding elements. ([#195](https://github.com/grafana/xk6-browser/pull/195))

- Possible issue with consecutive clicks on an element if the first click resulted in an exception. ([#219](https://github.com/grafana/xk6-browser/pull/219))

- `Page.waitForNavigation` becomes unresponsive if the browser or connection is closed or the user cancels the script. We now handle the extension shutting-down stage more robustly, and it shouldn't become unresponsive. ([#188](https://github.com/grafana/xk6-browser/pull/188))

- Passing arguments to the browser command now works correctly; see [this example](https://github.com/grafana/xk6-browser/blob/ae264530b6fbe0285ac9df9b0f4a907b460bb805/examples/browser_args.js). ([#206](https://github.com/grafana/xk6-browser/pull/206))

- `Page.waitForNavigation()` doesn't timeout anymore for navigation within the same document (e.g. via anchor links or the [History API](https://developer.mozilla.org/en-US/docs/Web/API/History_API)). ([#247](https://github.com/grafana/xk6-browser/pull/247)).

- Internal fixes that improve stability: [#217](https://github.com/grafana/xk6-browser/pull/217) and [#238](https://github.com/grafana/xk6-browser/pull/238).


## Improvements

- Errors for deprecated Chromium events are no longer printed by default, but can be seen when running in debug mode (`XK6_BROWSER_LOG=debug`). ([#251](https://github.com/grafana/xk6-browser/pull/251))

- Some error messages were enhanced to aid with debugging. ([#193](https://github.com/grafana/xk6-browser/pull/193))


## Internals

- The k6 dependency was updated to v0.36.0. It's recommended to upgrade to the latest [xk6 v0.6.1](https://github.com/grafana/xk6/releases/tag/v0.6.1) for building the extension. ([#202](https://github.com/grafana/xk6-browser/pull/202)).

- Code linting was fixed and expanded. ([#211](https://github.com/grafana/xk6-browser/pull/211), [#216](https://github.com/grafana/xk6-browser/pull/216), [#245](https://github.com/grafana/xk6-browser/pull/245)).

- Several code refactors and test additions that improve maintainability. ([#196](https://github.com/grafana/xk6-browser/pull/196), [#212](https://github.com/grafana/xk6-browser/pull/212), [#213](https://github.com/grafana/xk6-browser/pull/213) [#221](https://github.com/grafana/xk6-browser/pull/221), [#224](https://github.com/grafana/xk6-browser/pull/224), [#225](https://github.com/grafana/xk6-browser/pull/225), [#229](https://github.com/grafana/xk6-browser/pull/229))
